@@ -32,7 +32,6 @@ import Text.XML.HXT.DOM.Interface
 import Text.XML.HXT.Arrow.XmlArrow
 import Text.XML.HXT.Arrow.Edit                  ( canonicalizeAllNodes
                                                 , canonicalizeForXPath
-                                                , canonicalizeContents
                                                 , rememberDTDAttrl
                                                 , removeDocWhiteSpace
                                                 )
@@ -59,6 +58,20 @@ for all available options see module 'Text.XML.HXT.XmlState.SystemConfig'
 - @withValidate yes\/no@ :
   switch on\/off DTD validation. Only for XML parsed documents, not for HTML parsing.
 
+- @withSubstDTDEntities yes\/no@ :
+  switch on\/off entity substitution for general entities defined in DTD validation.
+  Default is @yes@.
+  Switching this option and the validation off can lead to faster parsing, in that case
+  reading the DTD documents is not longer necessary.
+  Only used with XML parsed documents, not with HTML parsing.
+
+- @withSubstHTMLEntities yes\/no@ :
+  switch on\/off entity substitution for general entities defined in HTML validation.
+  Default is @no@.
+  Switching this option on and the validation and substDTDEntities off can lead to faster parsing, in that case
+  reading the DTD documents is not longer necessary, HTML general entities are still substituted.
+  Only used with XML parsed documents, not with HTML parsing.
+
 - @withParseHTML yes\/no@ :
   switch on HTML parsing.
 
@@ -151,10 +164,28 @@ This requires packages \"hxt-curl\" and \"hxt-tagsoup\" to be installed
 >              , withHTTP              []
 >              ] "http://www.w3c.org/"
 
-read w3c home page (xhtml), validate and check namespaces, remove whitespace between tags, trace activities with level 2.
+read w3c home page (xhtml), validate and check namespaces, remove whitespace between tags,
+trace activities with level 2.
 HTTP access is done with Haskell HTTP package
 
-for minimal complete examples see 'Text.XML.HXT.Arrow.WriteDocument.writeDocument' and 'runX', the main starting point for running an XML arrow.
+> readDocument [ withValidate          no
+>              , withSubstDTDEntities  no
+>              ...
+>              ] "http://www.w3c.org/"
+
+read w3c home page (xhtml), but without accessing the DTD given in that document.
+Only the predefined XML general entity refs are substituted.
+
+> readDocument [ withValidate          no
+>              , withSubstDTDEntities  no
+>              , withSubstHTMLEntities yes
+>              ...
+>              ] "http://www.w3c.org/"
+
+same as above, but with substituion of all general entity refs defined in XHTML.
+
+for minimal complete examples see 'Text.XML.HXT.Arrow.WriteDocument.writeDocument'
+and 'runX', the main starting point for running an XML arrow.
 -}
 
 readDocument    :: SysConfigList -> String -> IOStateArrow s b XmlTree
@@ -205,8 +236,10 @@ readDocument'' src
           >>>
           ( if isAcceptedMimeType acceptedMimeTypes mimeType
             then ( ifA (fromLA hasEmptyBody)
-                   ( replaceChildren none )                                     -- empty response, e.g. in if-modified-since request
+                   ( replaceChildren none )               -- empty response, e.g. in if-modified-since request
                    ( ( parse $< getSysVar (theValidate              .&&&.
+                                           theSubstDTDEntities      .&&&.
+                                           theSubstHTMLEntities     .&&&.
                                            theIgnoreNoneXmlContents .&&&.
                                            theTagSoup               .&&&.
                                            theExpat
@@ -272,7 +305,7 @@ readDocument'' src
                                             (mi == mis || mis == "*")
                                           )
                                           || r
-        parse (validate, (removeNoneXml, (withTagSoup', withExpat')))
+        parse (validate, (substDTD, (substHTML, (removeNoneXml, (withTagSoup', withExpat')))))
             | not isXmlOrHtml           = if removeNoneXml
                                           then replaceChildren none             -- don't parse, if mime type is not XML nor HTML
                                           else this                             -- but remove contents when option is set
@@ -285,7 +318,7 @@ readDocument'' src
 
             | isXml                     = if withExpat'
                                           then parseXmlDocumentWithExpat
-                                          else parseXmlDocument (not validateWithRelax && validate)
+                                          else parseXmlDocument validate substDTD substHTML validateWithRelax
                                                                                 -- parse as XML
             | otherwise                 = this                                  -- suppress warning
 
@@ -369,28 +402,40 @@ readFromString config   = applyA ( arr $ readString config )
 -- ------------------------------------------------------------
 
 -- |
--- parse a string as HTML content, substitute all HTML entity refs and canonicalize tree
+-- parse a string as HTML content, substitute all HTML entity refs and canonicalize tree.
 -- (substitute char refs, ...). Errors are ignored.
 --
--- A simpler version of 'readFromString' but with less functionality.
--- Does not run in the IO monad
+-- This arrow delegates all work to the parseHtmlContent parser in module HtmlParser.
+--
+-- This is a simpler version of 'readFromString' without any options,
+-- but it does not run in the IO monad.
 
 hread                   :: ArrowXml a => a String XmlTree
 hread                   = fromLA $
                           parseHtmlContent                      -- substHtmlEntityRefs is done in parser
-                          >>>
-                          editNTreeA [isError :-> none]         -- ignore all errors
-                          >>>
-                          canonicalizeContents
+                          >>>                                   -- as well as subst HTML char refs
+                          editNTreeA [isError :-> none]         -- ignores all errors
 
--- |
--- parse a string as XML content, substitute all predefined XML entity refs and canonicalize tree
--- (substitute char refs, ...)
-
-xread                   :: ArrowXml a => a String XmlTree
-xread                   = parseXmlContent       -- substXmlEntityRefs is done in parser
+{- no longer neccesary, text nodes are merged in parser
                           >>>
                           canonicalizeContents
+-}
+
+-- ------------------------------------------------------------
+
+-- |
+-- parse a string as XML content, substitute all predefined XML entity refs and canonicalize tree
+-- This xread arrow delegates all work to the xread parser function in module XmlParsec
+
+xread                   :: ArrowXml a => a String XmlTree
+xread                   = parseXmlContent
+{- -- the old version, where the parser does not subst char refs and cdata
+xread                   = root [] [parseXmlContent]       -- substXmlEntityRefs is done in parser
+                          >>>
+                          canonicalizeContents
+                          >>>
+                          getChildren
+-}
 
 -- ------------------------------------------------------------
 
