@@ -122,6 +122,8 @@ initialParseConfig              = XIOPcfg
                                   , xioTagSoup                  = False
                                   , xioPreserveComment          = False
                                   , xioValidate                 = True
+                                  , xioSubstDTDEntities         = True
+                                  , xioSubstHTMLEntities        = False
                                   , xioCheckNamespaces          = False
                                   , xioCanonicalize             = True
                                   , xioIgnoreNoneXmlContents    = False
